//+------------------------------------------------------------------+
//|                                                          HMA.mq4 |
//|                      Copyright  2004, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+
#property  copyright "Copyright  2004, MetaQuotes Software Corp."
#property  link      "http://www.metaquotes.net/"
//---- indicator settings
#property  indicator_chart_window
#property  indicator_buffers 6
#property  indicator_color1  Blue
#property  indicator_color2  Red
#property  indicator_color3  Blue
#property  indicator_color4  Red
#property  indicator_color5  Aqua
#property  indicator_color6  Magenta
#property  indicator_width1  2
#property  indicator_width2  2
#property  indicator_width3  2
#property  indicator_width4  2

//---- indicator parameters

extern int  HMA_Period       = 40;
extern bool alertsOn         = false;
extern bool alertsOnCurrent  = true;
extern bool alertsMessage    = true;
extern bool alertsSound      = false;
extern bool alertsEmail      = false;
extern bool Show_ColoredBars = true;
extern int  arrowthickness   = 1;

double     ind_buffer0[];
double     ind_buffer1[];
double     upBar[];
double     dnBar[];
double     CrossUp[];
double     CrossDn[];
double     trend[];

int        draw_begin0;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
//---- indicator buffers mapping
   IndicatorBuffers(7);
   SetIndexBuffer(0,ind_buffer0); 
   SetIndexBuffer(1,ind_buffer1); 
   SetIndexBuffer(2,upBar);     
   SetIndexBuffer(3,dnBar);        
   SetIndexBuffer(4,CrossUp);   SetIndexStyle(4,DRAW_ARROW,0,arrowthickness); SetIndexArrow(4,221);
   SetIndexBuffer(5,CrossDn);   SetIndexStyle(5,DRAW_ARROW,0,arrowthickness); SetIndexArrow(5,222);
   SetIndexBuffer(6,trend);
   if (Show_ColoredBars)
   {
     SetIndexStyle(2,DRAW_HISTOGRAM); 
     SetIndexStyle(3,DRAW_HISTOGRAM); 
   }
   else
   {
     SetIndexStyle(2,DRAW_NONE);
     SetIndexStyle(3,DRAW_NONE);
   } 
   
   
     
   draw_begin0=HMA_Period+MathFloor(MathSqrt(HMA_Period));
   SetIndexDrawBegin(0,draw_begin0);
   IndicatorDigits(MarketInfo(Symbol(),MODE_DIGITS)+1);
//---- name for DataWindow and indicator subwindow label
   IndicatorShortName("HMA("+HMA_Period+")");
   SetIndexLabel(0,"Hull Moving Average");
//---- initialization done
   return(0);
  }
//+------------------------------------------------------------------+
//| Moving Averages Convergence/Divergence                           |
//+------------------------------------------------------------------+
int start()
{
   int i,counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-1);
//---- MA difference counted in the 1-st buffer
   for(i=limit; i>=0; i--) ind_buffer1[i]=iMA(NULL,0,MathFloor(HMA_Period/2),0,MODE_LWMA,PRICE_CLOSE,i)*2-iMA(NULL,0,HMA_Period,0,MODE_LWMA,PRICE_CLOSE,i);
   for(i=limit; i>=0; i--)
   {
      ind_buffer0[i]=iMAOnArray(ind_buffer1,0,MathFloor(MathSqrt(HMA_Period)),0,MODE_LWMA,i);
      trend[i] = trend[i+1];
         if (ind_buffer1[i]>ind_buffer0[i]) trend[i] =  1;
         if (ind_buffer1[i]<ind_buffer0[i]) trend[i] = -1;
         if (trend[i] == 1) { upBar[i] = High[i]; dnBar[i] = Low[i]; }
         if (trend[i] ==-1) { dnBar[i] = High[i]; upBar[i] = Low[i]; }
         
         CrossUp[i] = EMPTY_VALUE;
         CrossDn[i] = EMPTY_VALUE;
         if (trend[i]!= trend[i+1])
         if (trend[i] == 1)
               CrossUp[i] = ind_buffer0[i] - iATR(NULL,0,20,i)/2;
         else  CrossDn[i] = ind_buffer1[i] + iATR(NULL,0,20,i)/2;
   }      
   manageAlerts();
   return(0);
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] ==  1) doAlert(whichBar,"up");
         if (trend[whichBar] == -1) doAlert(whichBar,"down");
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  Symbol()+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+" HMA 2 lines trend changed to "+doWhat;
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(Symbol()+" HMA 2 lines",message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

